#!/usr/bin/python

# Google Latitude location updater 0.5
# Copyright (c) 2011 Anssi Kolehmainen <anssi@aketzu.net>
#
# Trivial modifications by MasterZap (twitter.com/MasterZap)
#  - Added "--once" option, so it can be called from e.g. Alarmed
#  - Changed to true GPS always
#  - Discard 'nan' accuracy inputs
#  - Only allow good accuracy points
#  - Warning: Requires fixed location for latitude.dat at /home/user/latitude/latitude.dat
#
# (Released under MIT Licence)
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.

#Requires http://code.google.com/p/google-api-python-client/ and
# python-location package

from apiclient.discovery import build

import httplib2
import pickle

from apiclient.discovery import build
from apiclient.oauth import FlowThreeLegged
from apiclient.ext.authtools import run
from apiclient.ext.file import Storage

import location
import gobject
import time
import socket
import os
import sys
import getopt

once = 0

class LocationData:
	lat=0; lng=0; alt=0; acc=0; alt=0; altacc=0; head=0; speed=0;

	prevlat=0
	prevlng=0
	prevalt=0
	prevacc=0

	prevupd=0
	prevacc=9999

	def push(self):
		global service
		if time.time() - self.prevupd < 60 and self.acc > self.prevacc:
			return True

		if self.lat == 0 or self.lng == 0:
			return True

		if self.prevlat == self.lat and self.prevlng == self.lng and time.time() - self.prevupd < 600:
			return True
 
		print "%s: Sending location update %f,%f (acc %f)" % (time.strftime('%Y-%m-%d %H:%M:%S'),self.lat,self.lng,self.acc)

		data = {
			"data": {
			  "kind": "latitude#location",
			  "latitude": self.lat,
			  "longitude": self.lng,
			  "accuracy": self.acc,
			  #"altitude": self.alt,
			  #"altitudeAccuracy": self.altacc,
			  #"heading": self.head,
			  #"speed": self.speed
			  }
		      }		
		#print data
		try:
			print service.currentLocation().insert(body=data).execute()
		except:
			print "Error updating location", sys.exc_info()[0]
			return True
     

		self.prevupd = time.time()
		self.prevacc = self.acc
		self.prevlat = self.lat
		self.prevlng = self.lng
		self.prevalt = self.alt

		if once:
			exit()

		return True
  
verbose=0
acclimit=150
curloc = LocationData()

def push_location():
	return curloc.push()

def update_loc(lat, lng, acc, alt, altacc, head, speed):
	global verbose
        global acclimit
        global once
	if verbose:
		print "%s: Got loc data %f,%f (acc %f) alt %f (%f), head %f, speed %f" % (time.strftime('%Y-%m-%d %H:%M:%S'),lat,lng,acc, alt, altacc, head, speed)

	# Skip the NaN's in accuracy
	if acc != acc:
		return

        # I don't care about data of low accuracy
	if acc > acclimit:
		return

	if altacc > 32000:
		altacc=0

	curloc.lat=lat
	curloc.lng=lng
	curloc.acc=acc
	curloc.alt=alt
	curloc.altacc=altacc
	curloc.head=head
	curloc.speed=speed
	return

def on_error(control, error, data):
    print "location error: %d..." % error
    #data.quit()

def on_changed(device, data):
    if not device:
        return
    if device.fix:
        if device.fix[1] & location.GPS_DEVICE_LATLONG_SET:
	    update_loc(device.fix[4], device.fix[5], device.fix[6]/1000, device.fix[7], device.fix[8], device.fix[9], device.fix[11])
            #lat, lon, eph,  altitude, epv, track, speed

def on_stop(control, data):
    print "quitting"
    #data.quit()

def start_location(data):
    data.start()
    return False

def usage():
	print "Location updater v0.2"
	print " -h  This help message"
	print " -d  Daemonize"
	print " -v  Verbose output"
	print " -p  Use commandline oauth response"
	print " -o  Once (send once, then terminate)"

def init():
	socket.setdefaulttimeout(10)

	loop = gobject.MainLoop()
	control = location.GPSDControl.get_default()
	device = location.GPSDevice()
	control.set_properties(preferred_method=location.METHOD_USER_SELECTED, #ACWP
			       preferred_interval=location.INTERVAL_120S)

	control.connect("error-verbose", on_error, loop)
	device.connect("changed", on_changed, control)
	control.connect("gpsd-stopped", on_stop, loop)

	gobject.idle_add(start_location, control)
	gobject.timeout_add(10000, push_location)

	loop.run()

def daemonize():
	pid = os.fork()
	if (pid == 0):
		os.setsid()
		pid = os.fork()
		if (pid == 0):
			os.umask(0)
		else:
			os._exit(0)
	else:
		os._exit(0)

	os.close(0);
	os.close(1);
	os.close(2);


def main(argv):
	global verbose, service, once
	try:
		opts, args = getopt.getopt(argv, "hdvpo", ["help", "verbose", "once"])
	except getopt.GetoptError:
		usage()
		sys.exit(2)
	
	storage = Storage('/home/user/latitude/latitude.dat')
	credentials = storage.get()
	if credentials is None or credentials.invalid == True:
		auth_discovery = build("latitude", "v1").auth_discovery()
		flow = FlowThreeLegged(auth_discovery,
			# You MUST have a consumer key and secret tied to a
			# registered domain to use the latitude API.
			#
			# https://www.google.com/accounts/ManageDomains
			consumer_key='aketzu.net',
			consumer_secret='tNthWntzHBcDpJ+9BnVX+/cO',
			user_agent='latitude-updater/1.0',
			domain='aketzu.net',
			scope='https://www.googleapis.com/auth/latitude',
			xoauth_displayname='Latitude updater',
			location='current',
			granularity='best'
			)
		credentials = run(flow, storage)

	http = httplib2.Http()
	http = credentials.authorize(http)

	service = build("latitude", "v1", http=http)


	for opt,arg in opts:
		if opt in ("-h", "--help"):
			usage()
			sys.exit(0)
		elif opt in ("-d"):
			daemonize()
		elif opt in ("-v"):
			verbose=1
		elif opt in ("-o", "--once"):
			once=1

	init()

main(sys.argv[1:])
